using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Windows;
using System.Windows.Media;
using SharpCharts.Base.Data;
using System.Xml.Serialization;
using SharpCharts.Base.Common;

namespace SharpCharts.Base.Chart
{
    public class SimpleProfile : ChartStyle
    {

        Base.Data.VolumeProfile barType = null;

        public SimpleProfile()
        {
            base.BarSpace = 5.0d;
            base.BarWidth = 100.0d;
            this.ProfileThickness = 1.0d;
            
            this.ProfileBrush = Brushes.DimGray;
        }

        public override void DrawBars(DrawingContext dc, double chartElementHeight, double chartElementWidth, Bars bars, int startIdx, int endIdx, double min, double max)
        {

            if (this.barType == null)
            {
                this.barType = bars.BarType as VolumeProfile;
                if (barType == null)
                    throw new InvalidCastException(string.Format("SimpleProfile.DrawBars : could not cast BarType - {0}", bars.BarType));


            }

            int maxPocVolume = 0;
            for (int i = endIdx; i >= startIdx; i--)
            {
                maxPocVolume = Math.Max(maxPocVolume, barType.ProfileItems[i].POCVolume);               
            }

            if (maxPocVolume == 0)
                return;

            double x = chartElementWidth - base.BarWidth;

            for (int i = endIdx; i >= startIdx; i--)
            {
                // draw the candles ***********************************
                IBar bar = bars.GetBar(i);

                double highY = base.CalculateYPoint(min, max, bar.High, chartElementHeight);
                double lowY = base.CalculateYPoint(min, max, bar.Low, chartElementHeight);
                double closeY = base.CalculateYPoint(min, max, bar.Close, chartElementHeight);
                double openY = base.CalculateYPoint(min, max, bar.Open, chartElementHeight);

                if (bar.Close > bar.Open)
                {
                    dc.DrawLine(base.UpWickPen, new Point(x - (base.BarSpace / 2), lowY), new Point(x - (base.BarSpace / 2), highY));
                    dc.DrawRectangle(base.UpBarBrush, base.UpWickPen,  new Rect(x - base.BarSpace, closeY, this.BarSpace, openY - closeY));
                }
                else
                {
                    dc.DrawLine(base.DownWickPen, new Point(x - (base.BarSpace / 2), lowY), new Point(x - (base.BarSpace / 2), highY));
                    dc.DrawRectangle(base.DownBarBrush, base.DownWickPen, new Rect(x - base.BarSpace, openY, base.BarSpace, closeY - openY));
                }

                // draw the profile ***********************************

                for (int j = 0; j < barType.ProfileItems[i].Profiles.Length; j++)
                {
                    if (barType.ProfileItems[i].Profiles[j] > 0)
                    {
                        double price = barType.ProfileItems[i].Max - (j * bars.Instrument.InstrumentDefination.TickSize);
                        double y = CalculateYPoint(min, max, price, chartElementHeight);
                        double x1 = base.BarWidth / maxPocVolume * barType.ProfileItems[i].Profiles[j];
                        dc.DrawLine(new Pen(this.ProfileBrush, this.ProfileThickness), new Point(x, y), new Point(x + x1, y));
                    }
                }

                x -= (base.BarWidth + base.BarSpace);
            }
        }


        
        public double ProfileThickness { get; set; }

        private Brush profileBrush;
        [XmlElement(ElementName = "ProfileBrush", Type = typeof(XamlSerializer<Brush>))]
        public Brush ProfileBrush
        {
            get { return profileBrush; }
            set 
            {
                profileBrush = value;
                if (profileBrush.CanFreeze)
                    profileBrush.Freeze();
            }
        }
    }
}
